"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const webpack_1 = __importStar(require("webpack"));
const html_webpack_plugin_1 = __importDefault(require("html-webpack-plugin"));
const mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
const copy_webpack_plugin_1 = __importDefault(require("copy-webpack-plugin"));
const fork_ts_checker_webpack_plugin_1 = __importDefault(require("fork-ts-checker-webpack-plugin"));
const eslint_webpack_plugin_1 = __importDefault(require("eslint-webpack-plugin"));
const webpackBar_1 = __importDefault(require("./common/webpackBar"));
const utils_1 = __importDefault(require("./utils"));
const ModuleFederationPlugin = webpack_1.container.ModuleFederationPlugin;
function hasTSConfig() {
    return fs_1.default.existsSync(path_1.default.resolve(process.cwd(), 'tsconfig.json'));
}
function addPlugins(basePlugins, plugins) {
    if (plugins) {
        plugins.forEach(function (plugin) {
            basePlugins.push(plugin);
        });
    }
    return basePlugins;
}
function getPlugins(rcb) {
    const basePlugins = [
        rcb.moduleFederationPlugin && new ModuleFederationPlugin(rcb.moduleFederationPlugin),
        new copy_webpack_plugin_1.default({
            patterns: [{
                    from: path_1.default.resolve('public'),
                    globOptions: { ignore: ['**/index.html'] }
                }]
        }),
        new webpack_1.default.DefinePlugin({
            'process.env': {
                NODE_ENV: JSON.stringify(utils_1.default.getEnv())
            }
        }),
        new eslint_webpack_plugin_1.default({
            extensions: ['.js', '.jsx', '.es6', '.ts', '.tsx', '.mjs'],
            eslintPath: require.resolve('eslint'),
            resolvePluginsRelativeTo: __dirname,
            baseConfig: utils_1.default.requireModule(require.resolve('./eslintrc'))
        }),
        new html_webpack_plugin_1.default({
            template: path_1.default.resolve('./public/index.html')
        }),
        new webpack_1.ProgressPlugin({
            handler(percentage, msg, ...args) {
                if (msg !== 'end') {
                    webpackBar_1.default.update(Math.floor(percentage * 10000) / 10000);
                }
            }
        })
    ];
    if (hasTSConfig()) {
        basePlugins.push(new fork_ts_checker_webpack_plugin_1.default({
            async: true
        }));
    }
    if (utils_1.default.isDev()) {
        addPlugins(basePlugins, [
            new mini_css_extract_plugin_1.default({
                filename: 'css/[name].css'
            })
        ]);
    }
    if (utils_1.default.isProduction()) {
        addPlugins(basePlugins, [
            new mini_css_extract_plugin_1.default({
                filename: 'css/[name].[contenthash:8].css'
            })
        ]);
    }
    if (rcb.webpackPluginConfig && rcb.webpackPluginConfig.length > 0) {
        return [basePlugins, ...rcb.webpackPluginConfig];
    }
    return basePlugins.filter(Boolean);
}
exports.default = getPlugins;
;
