"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const path_to_regexp_1 = require("path-to-regexp");
const paths_1 = __importDefault(require("../paths"));
const utils_1 = __importDefault(require("../utils"));
function readMockFile(req, res, next, mockConfig) {
    if (!mockConfig) {
        return new Error('没有找到mock配置');
    }
    try {
        const joinPath = paths_1.default.getJoinPath(mockConfig.rootDir, mockConfig.api, req.path);
        const reqUrl = path_1.default.join(mockConfig.api, req.url);
        if (mockConfig.whiteList
            && Array.isArray(mockConfig.whiteList)
            && mockConfig.whiteList.indexOf(reqUrl) > -1) {
            const isWhite = mockConfig.whiteList.some(white => {
                const matchExp = path_to_regexp_1.match(white);
                return matchExp(reqUrl);
            });
            if (isWhite) {
                return next();
            }
        }
        const mock = getMockResponse(joinPath);
        if (utils_1.default.isFunction(mock)) {
            const ret = mock(req, res);
            // 如果返回的是object那么自动处理这个函数 如果函数内处理了就不要返回值
            if (utils_1.default.isObject(ret)) {
                res.send(ret);
            }
        }
        else if (mock !== undefined && mock !== false) {
            res.send(mock);
        }
        else {
            next();
        }
    }
    catch (e) {
        utils_1.default.error(e);
    }
}
function readJsFile(filePath) {
    try {
        const parentDir = path_1.default.resolve(filePath, '../');
        const files = fs_1.default.readdirSync(parentDir);
        return files
            .filter(filename => /\.js$/.test(filename))
            .map(filename => path_1.default.resolve(parentDir, filename));
    }
    catch (e) {
        return [];
    }
}
function getMockResponse(joinPath) {
    let mock;
    try {
        clearRequireCache(`${joinPath}.js`);
        mock = utils_1.default.requireModule(`${joinPath}.js`);
    }
    catch (e) {
        // 动态路由 例如 /test/1 => /test/:id
        const jsFiles = readJsFile(joinPath);
        let i = 0;
        let fileIndex = -1;
        const len = jsFiles.length;
        while (i < len) {
            const jsFile = jsFiles[i];
            const matchExp = path_to_regexp_1.match(jsFile.replace(/\.js$/, ''), { sensitive: true });
            const realFileInfo = matchExp(joinPath);
            if (realFileInfo) {
                fileIndex = i;
                break;
            }
            i++;
        }
        if (fileIndex > -1) {
            clearRequireCache(jsFiles[fileIndex]);
            mock = require(jsFiles[fileIndex]); // 异步过程
        }
    }
    return mock;
}
function clearRequireCache(filePath) {
    const cacheMod = require.cache[filePath];
    // 防止内存泄漏
    if (cacheMod && cacheMod.parent) {
        const index = cacheMod.parent.children.indexOf(cacheMod);
        if (index > -1) {
            cacheMod.parent.children.splice(index, 1);
        }
    }
    delete require.cache[filePath];
}
exports.default = {
    middleware: function (options) {
        return function (req, res, next) {
            readMockFile(req, res, next, options);
        };
    }
};
