"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
function noop() {
    return null;
}
function getEnv() {
    return process.env.NODE_ENV;
}
function isDev() {
    return process.env.NODE_ENV === 'development';
}
function isProduction() {
    return process.env.NODE_ENV === 'production';
}
function isFunction(func) {
    return Object.prototype.toString.call(func) === '[object Function]';
}
function isObject(obj) {
    return Object.prototype.toString.call(obj) === '[object Object]';
}
function isString(str) {
    return Object.prototype.toString.call(str) === '[object String]';
}
function loggerPrefix(message, options) {
    const { name = 'info' } = options || {};
    console.log(`${chalk_1.default.blue(`react-build-cli:${name}`)} ${message}`);
}
function success(info) {
    loggerPrefix(`${chalk_1.default.green('✔')} ${chalk_1.default.blue(info)}`);
}
function logger(info) {
    loggerPrefix(chalk_1.default.green(info));
}
function warning(info) {
    loggerPrefix(chalk_1.default.yellow(info));
}
function error(info) {
    loggerPrefix(chalk_1.default.red(info));
}
function requireModule(modulePath) {
    return require(modulePath).default || require(modulePath);
}
function getWebpackProxyPath(webpackConfig) {
    var _a, _b;
    if (webpackConfig && webpackConfig.devServer && isObject(webpackConfig.devServer.proxy)) {
        const proxyPath = Object.keys(((_a = webpackConfig.devServer) === null || _a === void 0 ? void 0 : _a.proxy) || {});
        const resultMapStrings = [];
        proxyPath.forEach(key => {
            var _a;
            const proxy = (((_a = webpackConfig.devServer) === null || _a === void 0 ? void 0 : _a.proxy) || {})[key];
            if (isObject(proxy)) {
                resultMapStrings.push(`${key} => ${proxy.target}`);
            }
            else if (isString(proxy)) {
                resultMapStrings.push(`${key} => ${proxy}`);
            }
        });
        return resultMapStrings;
    }
    if (webpackConfig && webpackConfig.devServer && Array.isArray(webpackConfig.devServer.proxy)) {
        return (((_b = webpackConfig.devServer) === null || _b === void 0 ? void 0 : _b.proxy) || []).reduce((resultMapStrings, proxy) => {
            if (Array.isArray(proxy.path)) {
                resultMapStrings = [...resultMapStrings, ...proxy.path];
            }
            else if (isString(proxy.path)) {
                resultMapStrings.push(proxy.path || '');
            }
            return resultMapStrings;
        }, []);
    }
    return [];
}
function mergeBabelConfig(baseConfig, mergeConfig) {
    const basePresets = baseConfig.presets || [];
    const mergePresets = mergeConfig.presets || [];
    const basePlugins = baseConfig.plugins || [];
    const mergePlugins = mergeConfig.plugins || [];
    return Object.assign(Object.assign({}, baseConfig), { presets: [...basePresets, ...mergePresets], plugins: [...basePlugins, ...mergePlugins] });
}
exports.default = {
    chalk: chalk_1.default,
    loggerPrefix,
    logger,
    success,
    warning,
    error,
    noop,
    getEnv,
    isDev,
    isProduction,
    isFunction,
    isObject,
    isString,
    requireModule,
    getWebpackProxyPath,
    mergeBabelConfig
};
