import './babelRegister';

import webpack from 'webpack';
import { Options } from './start';
import WebpackDevServer from 'webpack-dev-server';

import utils from './utils';
import clearConsole from './clearConsole';
import getConfig from './getConfig';
import compilerErrorHandle from './compilerErrorHandle';
import openBrower from './openBrower';
import webpackBar from './common/webpackBar';

// 将devserver初始化log信息清除
class DevServer extends WebpackDevServer {
    constructor(...args: any) {
        super(...args);
    }
    logStatus() {
        return null;
    }
}

export default function runServer(options: Options) {
    const webpackConfig: webpack.Configuration & { devServer?: any } = getConfig(options).webpack;
    const devServerConfig = webpackConfig.devServer || {};

    const URL = `http://${devServerConfig.host}:${devServerConfig.port}`;
    const compiler = webpack(webpackConfig);

    const server: any = new DevServer(devServerConfig, compiler);
    let isFirstCompiler = true;

    compiler.hooks.beforeCompile.tap('beforeCompile', function () {
        if (isFirstCompiler) {
            webpackBar.update(0); // 优化：初始化时先展示0%时的webpack bar
            return;
        }

        clearConsole();
        utils.success(utils.chalk.green('Rebuild start...'));
    });

    compiler.hooks.afterDone.tap(
        'dev server',
        function (stats) {
            if (isFirstCompiler) {
                const proxyPaths = utils.getWebpackProxyPath(webpackConfig);

                utils.success(utils.chalk.green('Compiled successfully!'));
                utils.success(`${utils.chalk.green('Local:')}  ${URL}`);
                if (proxyPaths.length > 0) {
                    utils.success(`${utils.chalk.green('Proxy:')}  [${proxyPaths.join(', ')}]`);
                }
                

                isFirstCompiler = false;
                devServerConfig?.open && openBrower(URL);
            }
            else {
                utils.success(utils.chalk.green('Compiled successfully!'));
            }

            compilerErrorHandle(stats, null, isFirstCompiler);
        }
    )

    if (!devServerConfig.port || !devServerConfig.host) {
        utils.error(`port: ${devServerConfig.port}, host: ${devServerConfig.host}`);
        process.exit(0);
    }

    server
        .start()
        .catch((e: any) => {
            if (e) {
                utils.error(e.message);
                process.exit(0);
            }
        });
}
